@@ -1,4 +1,4 @@
-#    $Id: Nominet.pm,v 1.13 2015/11/04 16:23:31 pete Exp $
+#    $Id: Nominet.pm,v 1.18 2017/09/05 14:25:02 pete Exp $
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ use constant EPP_XMLNS	=> 'urn:ietf:params:xml:ns:epp-1.0';
 use vars qw($Error $Code $Message);
 
 BEGIN {
-	our $VERSION = '0.02_04';
+	our $VERSION = '0.03_01';
 }
 
 # file-scoped lexicals
@@ -44,19 +44,19 @@ my $Debug     = 0;
 
 =head1 Name
 
-Net::EPP::Registry::Nominet - a simple client interface to Nominet EPP
-jobs
+Net::EPP::Registry::Nominet - a simple client interface to the Nominet EPP
+service
 
 =head1 Synopsis
 
-	#!/usr/bin/perl
 	use strict;
+	use warnings;
 	use Net::EPP::Registry::Nominet;
 
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $epp->get_error);
+	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
 
 	my $dom = 'foo.co.uk';
 	
@@ -80,8 +80,8 @@ jobs
 
 L<Nominet|http://www.nominet.org.uk/> is the organisation in charge of
 domain names under the .uk TLD.  Historically it used cryptographically
-signed email communications with registrars to provision domains (and
-still does). More recently (since 2010) it has instituted an EPP system
+signed email communications with registrars to provision domains.
+More recently (since 2010) it has instituted an EPP system
 which is sufficiently different from standard EPP that none of the
 standard modules will work seamlessly with it.
 
@@ -102,10 +102,10 @@ END {}
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $epp->get_error);
+	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
 
 The constructor for C<Net::EPP::Registry::Nominet> has the same
-general form as the one for C<Net::EPP::Simple>, but with the following
+general form as the one for L<Net::EPP::Simple>, but with the following
 exceptions:
 
 =over
@@ -127,6 +127,9 @@ renewal command. It must be an integer between 1 and 10 inclusive (but
 note that renewing for 10 years pre-expiry will always fail because
 Nominet prohibits it).
 
+=item * C<login_opt> is a hashref of options to be passed directly
+through as the third optional argument to L<login()|/Login>.
+
 =item * There is no facility for a config file but this may be added in
 future versions.
 
@@ -207,7 +210,7 @@ sub new {
 
 	# Login
 	unless (defined $params{login} and $params{login} == 0) {
-		$self->login ($params{user}, $params{pass});
+		$self->login ($params{user}, $params{pass}, $params{login_opt});
 	}
 
 	# If there was an error in the constructor, there's no point
@@ -225,16 +228,31 @@ The client can perform a standalone EPP Login if required.
 		or die ("Could not login: ", $epp->get_reason);
 
 The optional third argument, C<$opt_ref>, is a hash ref of login
-options. Currently the only supported option is 'nominet_schemas' which
-should be set to a true value if the user requires the old, deprecated
-Nominet EPP schemas. These were removed from the Live systems before
-August 2013, so should not be retained in production code.
+options. Currently the only supported option is 'tag_list' which
+should be set to a true value if the user needs to use the
+L<list_tags|/List Tags>
+method. Nominet operates a number of
+L<mutually exclusive schemas|http://registrars.nominet.uk/registration-and-domain-management/schemas-and-namespace-uris>
+so that the user needs to login again to perform different tasks. At present
+this module only supports two sets: the standard tasks and the tag list.
 
 =cut
 
 sub login {
 	my ($self, $user, $pass, $options) = @_;
 
+	unless (defined $user) {
+		$Error = 'No username (tagname) supplied';
+		carp ($Error);
+		return;
+	}
+
+	unless (defined $pass) {
+		$Error = 'No password supplied';
+		carp ($Error);
+		return;
+	}
+
 	# Set login frame
 	my $login = Net::EPP::Frame::Command::Login->new;
 
@@ -243,39 +261,36 @@ sub login {
 	$login->version->appendText($EPPVer);
 	$login->lang->appendText($EPPLang);
 
-	my $objects = $self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'objURI');
-	#while (my $object = $objects->shift) {
-	#for my $ns ('nom-account', 'nom-contact', 'nom-domain',
+	my @ns   = ();
+	my @svcs = ();
+	my $baseuri = 'http://www.nominet.org.uk/epp/xml';
 
-
-	if ($options->{'nominet_schemas'}) {
-		# Deprecated
-		for my $ns ('nom-domain', 'nom-notifications') {
-			my $el = $login->createElement('objURI');
-			$el->appendText("http://www.nominet.org.uk/epp/xml/$ns-$NSVer");
-			$login->svcs->appendChild($el);
-		}
-		my $foo = $login->createElement('objURI');
-		$foo->appendText("urn:ietf:params:xml:ns:host-1.0");
-		$login->svcs->appendChild($foo);
+	if ($options->{'tag_list'}) {
+		push @ns, 'nom-tag';
 	} else {
-		# Standard schemas and extensions
-		for my $ns ('epp', 'eppcom', 'domain', 'host', 'contact', 'secDNS') {
-			my $el = $login->createElement('objURI');
-			my $ver = $EPPVer;
-			$ver = 1.1 if $ns eq 'secDNS';
-			$el->appendText("urn:ietf:params:xml:ns:$ns-$ver");
-			$login->svcs->appendChild($el);
-		}
-		# Extensions go here
+		push @ns, qw/epp eppcom domain host contact secDNS/;
+		push @svcs, qw/domain-nom-ext-1.2 contact-nom-ext-1.0
+			std-notifications-1.2 std-warning-1.1 std-contact-id-1.0
+			std-release-1.0 std-handshake-1.0 nom-abuse-feed-1.0
+			std-fork-1.0 std-list-1.0 std-locks-1.0 std-unrenew-1.0
+			nom-direct-rights-1.0/;
+	}
+	# Standard schemas and extensions
+	for my $ns (@ns) {
+		my $el = $login->createElement('objURI');
+		my $ver = $EPPVer;
+		$ver = 1.1 if $ns eq 'secDNS';
+		my $text = "urn:ietf:params:xml:ns:$ns-$ver";
+		$text = "$baseuri/$ns-$ver" if $ns =~ /-/;
+		$el->appendText($text);
+		$login->svcs->appendChild($el);
+	}
+	# Extensions go here
+	if (scalar @svcs) {
 		my $ext = $login->createElement('svcExtension');
-		for my $ns (qw/domain-nom-ext-1.2 contact-nom-ext-1.0
-		std-notifications-1.2 std-warning-1.1 std-contact-id-1.0
-		std-release-1.0 std-handshake-1.0 nom-abuse-feed-1.0
-		std-fork-1.0 std-list-1.0 std-locks-1.0 std-unrenew-1.0
-		nom-direct-rights-1.0/) {
+		for my $ns (@svcs) {
 			my $el = $login->createElement('extURI');
-			$el->appendText("http://www.nominet.org.uk/epp/xml/$ns");
+			$el->appendText("$baseuri/$ns");
 			$ext->appendChild($el);
 		}
 		$login->svcs->appendChild($ext);
@@ -294,7 +309,7 @@ sub login {
 
 =head1 Availability checks
 
-The availability checks work similarly to C<Net::EPP::Simple> except
+The availability checks work similarly to L<Net::EPP::Simple> except
 that they return an array with three elements. The first element is the
 availability indicator as before (0 if provisioned, 1 if available,
 undef on error) and the second element is the abuse counter which shows
@@ -380,6 +395,18 @@ be post-expiry.
 
 sub renew {
 	my ($self, $renew) = @_;
+	unless (defined $renew) {
+		carp "No argument provided";
+		return undef;
+	}
+	unless (ref $renew and ref $renew eq 'HASH') {
+		carp "Argument to renew is not a hash reference";
+		return undef;
+	}
+	unless ($renew->{name}) {
+		carp "Argument to renew has no 'name' field";
+		return undef;
+	}
 	my $domain = $renew->{name};
 	my $expiry = $renew->{cur_exp_date};
 	my $years  = $renew->{period};
@@ -532,7 +559,7 @@ registration.
 			'nsname0'  => "ns1.bar.co.uk",
 			'nsname1'  => "ns2.bar.co.uk"
 		},
-		sedDNS       => [
+		secDNS       => [
 			{
 				keyTag     => 123,
 				alg        => 5,
@@ -812,18 +839,18 @@ eg:
 		'add'          => {
 			secDNS => [{ 
 				keyTag     => 25103,
-				alg	       => 5,
+				alg        => 5,
 				digestType => 1,
 				digest     => '8A9CEBB665B78E0142F1CEF47CC9F4205F600685'
 			}]
-		}
+		},
 		'rem'          => {}
 	};
 
 The extension fields can only be set outside of the add, rem and
-chg fields. The supported extensions in this module are: auto-bill,
-auto-period, next-bill, next-period, renew-not-required and notes. All
-of these are scalars aside from notes which is an array ref.
+chg fields. The supported extensions in this module are: C<auto-bill>,
+C<auto-period>, C<next-bill>, C<next-period>, C<renew-not-required> and
+C<notes>. All of these are scalars aside from C<notes> which is an array ref.
 
 C<update_domain()> returns undef on failure, 1 on success.
 
@@ -941,7 +968,7 @@ changes like this:
 	};
 	my $res = $epp->update_contact ($changes) or die $epp->get_reason;
 
-Note that this differs from the syntax of C<Net::EPP::Simple> where that
+Note that this differs from the syntax of L<Net::EPP::Simple> where that
 takes the stock C<add>, C<rem> and C<chg> elements.
 
 It returns undef on failure, 1 on success.
@@ -1190,6 +1217,12 @@ sub _domain_infData_to_hash {
 	return $hash;
 }
 
+sub _tag_infData_to_hash {
+	my ($self, $infData) = @_;
+	return $self->_node_to_hash ($infData, ['registrar-tag',
+		'name', 'handshake', 'trad-name']);
+}
+
 sub _clean_addr {
 	my ($self, $infData) = @_;
 	# Remove the whitespace text nodes from the addresses as
@@ -1280,6 +1313,55 @@ sub list_domains {
 	return $domlist;
 }
 
+=head2 List Tags
+
+When transferring domains it may be useful to have a list of possible
+tag names. This method returns the full list of tags as an array ref.
+Each entry in the arrayref is itself a hashref with these keys:
+
+=over
+
+=item C<registrar-tag> is the tag name to use in release actions, etc.
+
+=item C<name> is the name of the registrar for display purposes
+
+=item C<trad-name> is the trading name of the registrar (may be empty
+string)
+
+=item C<handshake> is "Y" if they require handshakes on transfer
+or "N" otherwise
+
+=back
+
+	my $taglist = $epp->list_tags;
+
+It accepts no arguments and returns undef on error.
+
+Note that you must have passed the C<tag_list> option to L<login()|/Login>
+in order to use this method.
+
+=cut
+
+sub list_tags {
+	my $self = shift;
+	my $type = 'tag';
+	my @spec = $self->spec ($type);
+	my $frame = Net::EPP::Frame::Command::Info->new;
+	my $name = $frame->createElement ('tag:list');
+	$name->setAttribute ("xmlns:$type", $spec[1]);
+	$frame->getCommandNode->appendChild ($name);
+
+	my $response = $self->_send_frame($frame) or return undef;
+	if ($Code != 1000) { return undef; }
+
+	my $infData = $response->getNode($spec[1], 'listData');
+	my $taglist = [];
+	for my $node ($infData->childNodes) {
+		push @$taglist, $self->_tag_infData_to_hash ($node);
+	}
+	return $taglist;
+}
+
 =head1 Hello
 
 EPP allows the use of a "hello" operation which effectively tests that
@@ -1293,7 +1375,7 @@ The hello method takes no arguments. It returns 1 on success, undef
 otherwise.
 
 This performs much the same function as the ping method of
-Net:EPP::Simple (which could be used instead) but provides more
+L<Net::EPP::Simple> (which could be used instead) but provides more
 extensive error handling.
 
 =cut
@@ -1389,6 +1471,11 @@ sub spec {
 			"http://www.nominet.org.uk/epp/xml/std-release-1.0",
 			"http://www.nominet.org.uk/epp/xml/std-release-1.0 std-release-1.0.xsd");
 	}
+	if ($type eq 'tag') {
+		return ($type,
+			"http://www.nominet.org.uk/epp/xml/nom-tag-$EPPVer",
+			"http://www.nominet.org.uk/epp/xml/nom-tag-$EPPVer nom-tag-$EPPVer.xsd");
+	}
 }
 
 =head2 valid_voice
@@ -1525,13 +1612,9 @@ sub _send_frame {
 
 =over
 
-=item * The poll, fork, handshake, lock, tag list and reseller operations
+=item * The poll, fork, handshake, lock and reseller operations
 are not yet supported.
 
-=item * Retrieving contact info can generate warnings from the
-superclass. Ideally this processing should be done locally (within this
-class) to avoid these warnings.
-
 =item * Much more extensive tests should be performed.
 
 =back
@@ -1558,7 +1641,7 @@ Pete Houston <cpan@openstrike.co.uk>
 
 =head1 Licence
 
-This software is copyright (c) 2013-2015 by Pete Houston. It is released
+This software is copyright (c) 2013-2017 by Pete Houston. It is released
 under the Artistic Licence (version 2) and the
 GNU General Public Licence (version 2).
 
